package tests.part1;
/*********************************************************************************************
//* Filename:		Test6_String_SystemKey_Headerless_Mode.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.SingleJobDecryptionMode;
import com.clear.cryptosystem.CLEARMode.SingleJobEncryptionSysGenKeyMode;
import com.clear.cryptosystem.CLEARResult;
import com.clear.cryptosystem.CLEARString;

import tools.EvaluateResult;
import tools.InspectResult;
import tools.Stopwatch;

public class Test6_String_SystemKey_Headerless_Mode {

	// TEST DATA - HARD CODED
	private static final String _CLEARTEXT_TEST_DATA 			= "The quick brown fox jumps over the lazy dog!!  Some UTF-16 based 中国人 characters";
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	
	/**
	 * UNIT TEST BLOCK
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test() throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test6_String_SystemKey_Headerless_Mode.class.getCanonicalName() + "]");
		
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// INITIALIZE "STRING" INTERFACE
		CLEARString cs = CLEAR.clearString();		// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)
		
		
		// ENCRYPT A STRING - SYSTEM GENERATED KEY
		sw.setTimeMark(1);
		CLEARResult cr = cs.encrypt(_CLEARTEXT_TEST_DATA, CLEAR.STRENGTH_512_BIT, SingleJobEncryptionSysGenKeyMode.SINGLE_JOB_SYSTEM_GEN_KEY_HEADERLESS);
		sw.setTimeMark(2);
		
		
		// INSEPCT OUTPUT OF ENCRYPTION
		InspectResult.inspectEncryptionResultsString(cr, sw, _CLEARTEXT_TEST_DATA, Test6_String_SystemKey_Headerless_Mode.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// DECRYPT A STRING - WITH THE SYSTEM-GENERATED KEYSTRING PRODUCED BY ENCRYPTION
		sw.setTimeMark(3);
		cr = cs.decrypt(cr.getCipherTextString(), cr.getKeyMaterial(), SingleJobDecryptionMode.SINGLE_JOB_KEY_HEADERLESS);
		sw.setTimeMark(4);
		
		
		// INSPECT OUTPUT OF DECRYPTION
		InspectResult.inspectDecryptionResultsString(cr, sw, _CLEARTEXT_TEST_DATA, Test6_String_SystemKey_Headerless_Mode.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// EVALUATE DECRYPTION RESULT
		pass = EvaluateResult.evaluateStringDecryptionResult(cr, _CLEARTEXT_TEST_DATA, Test6_String_SystemKey_Headerless_Mode.class.getCanonicalName());
		
		return pass;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	 * @return void
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			test();
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
