package tests.part3;
/*********************************************************************************************
//* Filename:		Test2_Streaming_SystemKey_Headerless_Mode.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.StreamingDecryptionMode;
import com.clear.cryptosystem.CLEARMode.StreamingEncryptionSysGenKeyMode;
import com.clear.cryptosystem.CLEARResult;
import com.clear.cryptosystem.CLEARStream;

import tools.EvaluateResult;
import tools.InspectResult;
import tools.Stopwatch;

public class Test2_Streaming_SystemKey_Headerless_Mode {

	private static String _SYMMETRIC_KEY_EXCHANGE;  					 // SET DURING TRANSMISSION PHASE
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	
	/**
	 * UNIT TEST BLOCK
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test() throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test2_Streaming_SystemKey_Headerless_Mode.class.getCanonicalName() + "]");
		
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// GENERATE MOCKING DATA - SRC
		List<byte[]> src = generateMockingDataSRC();  	// GET MOCKING DATA
		
		
		// ENCRYPT "PACKETS" OF DATA TO BE STREAMED -> (I.E;  SOCKET SERVER, ETC.)
		sw.setTimeMark(1);
		ArrayDeque<byte[]> encryptedPackets = encryptPackets(src);
		sw.setTimeMark(2);
		
		
		// INSEPCT OUTPUT OF ENCRYPTION
		InspectResult.inspectEncryptionResultsStreaming(src, encryptedPackets, sw, _SYMMETRIC_KEY_EXCHANGE, Test2_Streaming_SystemKey_Headerless_Mode.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// DECRYPT "PACKETS" OF DATA FROM STREAM 
		sw.setTimeMark(3);
		List<byte[]> decryptedPackets = decryptPackets(encryptedPackets);
		sw.setTimeMark(4);
		
		
		// INSPECT OUTPUT OF DECRYPTION
		InspectResult.inspectDecryptionResultsStreaming(src, decryptedPackets, sw, Test2_Streaming_SystemKey_Headerless_Mode.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// EVALUATE DECRYPTION RESULT
		pass = EvaluateResult.evaluateUnitTestResultStreaming(src, decryptedPackets, Test2_Streaming_SystemKey_Headerless_Mode.class.getCanonicalName());
		
		
		return pass;
	}

	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> ENCRYPTION IMPLEMENTATION BELOW THIS LINE : 																																								  				  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - ENCRYPT STREAMING DATA PACKETS
	 * @param List<byte[]> src
	 * @return ArrayDeque<byte[]> 
	*/
	private static ArrayDeque<byte[]> encryptPackets(List<byte[]> src) {
		
		// INITIALIZE "STREAM" INTERFACE
		CLEARStream cs = CLEAR.clearStream();			// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)

		
		// INSTANTIATE COLLECTIONS / MOCKING DATA
		ArrayDeque<byte[]> packets = new ArrayDeque<byte[]>();
		
		
		// VARIABLES TO CONTROL STREAMING ENCRYPTION JOB
		int c = 0;
		long jobID = 0;
		boolean terminate = false;
		String keyString = null;
		CLEARResult cr = null;
		
		int strength = CLEAR.STRENGTH_512_BIT;
//		int strength = CLEAR.STRENGTH_1024_BIT;
		
		
		// LOOP THROUGH STREAMING DATA (SYNTHESIZED "PACKET" DATA)
		for (byte[] data : src) {
			
			
			// CHECK FOR END OF PACKET STREAM - CLOSE ENCRYPTION JOB AT END OF SRC DATA
			if (c == (src.size() -1)) { 
				terminate = true; // CAUSES ENCRYPTION TO END
			}
			
			
			// DO ENCRYPTION!  - FIRST PACKET INITIALIZES JOB - SUBSEQUENT PACKETS CONTINUE ON "JOB-ID"
			if (c == 0) {
				
				// BEGIN A NEW STREAMING JOB (FIRST PACKET) - PERFORMS INIT
				cr = cs.encryptStart(data, strength, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY_HEADERLESS, terminate);  // ENCRYPT A PACKET
				jobID = cr.getJobNumber(); // KEEP TRACK OF STREAMING JOB NUMBER FOR NEXT ITERATION
			}
			else {
				
				// CONTINUE EXISTING STREAMING BASED ON ** JOB-ID ** - (NN PACKETS) 
				cr = cs.encryptContinue(data, jobID, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY_HEADERLESS, terminate);
			}
			
			// PARSE SYSTEM-GENERATED KEY MATERIAL FROM INITIALIZATION SEGMENT
			keyString = (keyString == null ? cr.getKeyMaterial() : keyString);
			
			
			// PERFORM SYMMETRIC PRIVATE-KEY EXCHANGE - THIS UNIT (ALICE) NEEDS TO TRANSMIT A KEY TO BOB
			transmitKeyExchange(keyString);  // TRANSMIT KEY TO BOB
			
			
			// CAPTURE ENCRYPTED PACKET DATA TO SEND ONWARD TO RECEIVER
			packets.add(cr.getCipherText());
			c++;
		}
		
		return packets;
	}
	
	/**
	 * ATOMIC SUBROUTINE - SYNTHESIZE A KEY EXCHANGE
	 * @param keyString
	*/
	private static void transmitKeyExchange(String keyString) {
		
		if (_SYMMETRIC_KEY_EXCHANGE == null) {
			_SYMMETRIC_KEY_EXCHANGE = keyString;
		}
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> DECRYPTION IMPLEMENTATION BELOW THIS LINE : 																																								  				  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - DECRYPT STREAMING DATA PACKETS
	 * @parm ArrayDeque<byte[]> packets
	 * @return List<byte[]>
	*/
	private static List<byte[]> decryptPackets(ArrayDeque<byte[]> packets) {
		
		// PULL FROM STREAMING FAUX NETWORK SOURCE - DECRYPT EACH BYTE AS WE RECEIVE THEM.  
		// ** SIMULATES RECEIVER **
		List<byte[]> decryptedPackets = new ArrayList<byte[]>();
		
		
		// INITIALIZE "STREAM" INTERFACE
		CLEARStream cs = CLEAR.clearStream();			// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)
		
		
		// VARIABLES TO CONTROL STREAMING DECRYPTION JOB
		int p = 0;
		long jobID = 0;
		int totalReceived = packets.size();
		boolean terminate = false; // reset flag
		CLEARResult cr = null;
		
		
		// PERFORM SYMMETRIC PRIVATE-KEY EXCHANGE - THIS UNIT (BOB) NEEDS TO RECEIVE A KEY FROM ALICE
		String keyString = _SYMMETRIC_KEY_EXCHANGE;
		
		
		// LOOP THROUGH STREAMING DATA (SYNTHESIZED "PACKET" DATA)
		while (!packets.isEmpty()) {
			
			
			byte[] packet = packets.removeFirst();  // TAKE NEXT AVAILABLE PACKET OF ENCRYPTED DATA (CIPHERTEXT)
			
			
			if (p == (totalReceived -1)) { 
				terminate = true; // CAUSES DECRYPTION TO END (GENERATES HMAC SIGNATURE WHEN IN HMAC MODE)
			}
			
			
			// DO DECRYPTION!  - FIRST PACKET INITIALIZES JOB - SUBSEQUENT PACKETS CONTINUE ON "JOB-ID"
			if (p == 0) {
				
				// BEGIN A NEW STREAMING JOB (FIRST PACKET) - PERFORMS INIT
				cr = cs.decryptStart(packet, keyString, StreamingDecryptionMode.STREAMING_KEY_HEADERLESS, terminate);
				jobID = cr.getJobNumber();  // KEEP TRACK OF STREAMING JOB NUMBER FOR NEXT ITERATION
			}
			else {
				
				// CONTINUE EXISTING STREAMING BASED ON ** JOB-ID ** - (NN PACKETS) 
				cr = cs.decryptContinue(packet, jobID, StreamingDecryptionMode.STREAMING_KEY_HEADERLESS, terminate);
			}
			
			
			// CAPTURE DECRYPTED (CLEARTEXT) PACKET(S)
			decryptedPackets.add(cr.getClearText());
			
			p++;
		}
		
		return decryptedPackets;
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> HELPER FUNCTIONS BELOW THIS LINE : 																																														  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - SIMULATE TEMP SENSOR DATA
	 * @return List<byte[]>
	*/
	private static List<byte[]> generateMockingDataSRC() {
		
		int sample_size = 100;
		
		// CREATE SAMPLE IIOT DATA SOURCE
		Random r = new Random();
		int max = 91;
		int min = 64;
		List<byte[]> src = new ArrayList<byte[]>();
		for (int i = 0; i < sample_size; i++) {
			String data = new String("TEMP [78] DEGREES");
			int adjust = r.nextInt(max - min) + min;
			data = data.replace("78", "" + adjust);
			src.add(data.getBytes());
		}
		
		return src;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	 * @return void
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			test();
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
