package tests.part4;
/*********************************************************************************************
//* Filename:		Test4_Keytool_HyperKey_Generate_User_Token.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARKeyTool;
import com.clear.cryptosystem.CLEARLicenseInfo;

import tools.EvaluateResult;
import tools.InspectResult;
import tools.LoadMFA;
import tools.Stopwatch;

public class Test4_Keytool_HyperKey_Generate_User_Token {
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	/**
	 * UNIT TEST BLOCK
	 * @param byte[] mfaMaterial
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test(byte[] mfaMaterial) throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test4_Keytool_HyperKey_Generate_User_Token.class.getCanonicalName() + "]");
		
		// ENCRYPT SOME DATA WITH THE KEY THAT WAS CREATED
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// MAKE A STANDARD-KEY
		CLEARKeyTool keyTool = CLEAR.clearKeyTool();
		
		
		// GENERATE USER TOKEN
		sw.setTimeMark(1);
		String userToken = keyTool.generateHyperKeyUserToken(mfaMaterial);
		sw.setTimeMark(2);
		
		
		if (userToken != null) {
			pass = true;
		}
		
		if (InspectResult.is_DISPLAY_RESULTS_TO_CONSOLE()) {
			System.out.println(userToken);
			System.out.println("Generated User Token: " + sw.compareTimeMilli(1, 2) + " milliseconds");
		}
		
		return pass;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			
			if ((args != null) && (args.length == 1) && (args[0] != null)) {
				
				File mfa = new File(args[0]);
				
				if ((mfa != null) && ((mfa.exists()))) {
					
					// MAKE A BACKUP COPY OF THE INPUT FILE SO THAT IT CAN BE USED DURING TESTING
					byte[] mfaMaterial = LoadMFA.load(mfa.getAbsolutePath());
					
					test(mfaMaterial); 	 // EXECUTE UNIT TEST
				}
				else {
					System.out.println("Could not locate MFA material:  " + args[0]);
				}
			}
			else {
				System.out.println("This unit test requires program input parameters");
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
