package tests.part4;
/*********************************************************************************************
//* Filename:		Test5_KeyTool_HyperKey_Add_User_ACL.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARKeyTool;
import com.clear.cryptosystem.CLEARLicenseInfo;

import tools.EvaluateResult;
import tools.FileIO;
import tools.InspectResult;
import tools.LoadMFA;

public class Test5_KeyTool_HyperKey_Add_User_ACL {

	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	/**
	 * UNIT TEST BLOCK
	 * @param String userToken
	 * @param byte[] ownerMFAMaterial
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test(String userToken, byte[] ownerMFAMaterial) throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test5_KeyTool_HyperKey_Add_User_ACL.class.getCanonicalName() + "]");
		
		// MAX STRENGTH = MAXIMUM 7 USERS + OWNER.   ALL OTHER STRENGTHS = MAXIMUM 13 USERS + OWNER
		
		int strength = CLEAR.STRENGTH_512_BIT;
//		int strength = CLEAR.STRENGTH_1024_BIT;
//		int strength = CLEAR.STRENGTH_2048_BIT;
//		int strength = CLEAR.STRENGTH_5120_BIT;
//		int strength = CLEAR.STRENGTH_10240_BIT;
		
		
		
		// MAKE A STANDARD-KEY
		CLEARKeyTool keyTool = CLEAR.clearKeyTool();
		String usk = keyTool.genKey(strength);
		
		
		// CONVERT STANDARD KEY TO A HYPER-KEY
		String hyperKey = keyTool.createHyperKey(usk, ownerMFAMaterial);
		
		// ADD USER TO HYPER-KEY
		String updatedHyperKey = keyTool.addHyperKeyACL(hyperKey, userToken, ownerMFAMaterial, 10, 20, 30, 45);
		
		
		// DECRYPT WITH THE UPDATED HYPERKEY => CAN ONLY BE DONE BY USER
		
		System.out.println(updatedHyperKey);
		
		return pass;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			
			if ((args != null) && (args.length == 2) && (args[0] != null)) {
				
				File mfa = new File(args[0]);
				File userACLToken = new File(args[1]);
				
				if ((mfa != null) && (mfa.exists() && (userACLToken != null) && (userACLToken.exists()))) {
					
					// MAKE A BACKUP COPY OF THE INPUT FILE SO THAT IT CAN BE USED DURING TESTING
					byte[] ownerMFAMaterial = LoadMFA.load(mfa.getAbsolutePath());
					
					String userToken = FileIO.getStringFromTextFile(userACLToken);
					
					test(userToken, ownerMFAMaterial); 	 // EXECUTE UNIT TEST
					
				}
				else {
					System.out.println("Could not locate OWNER MFA material or USER TOKEN:  " + args[0] + " - " + args[1]);
				}
			}
			else {
				System.out.println("This unit test requires program input parameters");
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
