package tests.part4;
/*********************************************************************************************
//* Filename:		Test6_HyperKey_Triple_User_Test.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARFile;
import com.clear.cryptosystem.CLEARKeyTool;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.StreamingDecryptionMode;
import com.clear.cryptosystem.CLEARMode.StreamingEncryptionSysGenKeyMode;
import com.clear.cryptosystem.CLEARMode.StreamingEncryptionUserSuppliedKeyMode;
import com.clear.cryptosystem.CLEARResult;

import tools.EvaluateResult;
import tools.FileIO;
import tools.InspectResult;
import tools.LoadMFA;

public class Test6_HyperKey_Triple_User_Test {
	
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	
	/**
	 * UNIT TEST BLOCK
	 * @param String userToken1
	 * @param String userToken2
	 * @param byte[] ownerMFAMaterial
	 * @param File dataToEncrypt
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test(String userToken1, String userToken2, byte[] ownerMFAMaterial, File dataToEncrypt) throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test6_HyperKey_Triple_User_Test.class.getCanonicalName() + "]");
		
		File workingDIR = dataToEncrypt.getParentFile();
		
		// PRE-REQUISITES - Note:  Pre-Requisites must be done externally
		
		// 1.  User1, and User2 each have their own actual license file (CLEAR.LIC) and unique MFA Material - sorted separately
		
		// 2.  User1 and User2 generate HyperKey Tokens via CLI
		
		// 3.  Owner user is an "internal user" such that this test can be run in debug mode - Note:  This test will not work correctly in EXTERNAL-TESTER
		
		// 4.  Owner user (IDE ENV) is pointing to the CLEAR.LIC in the owner user folder
		
		
		// PART 1 - OWNER ENCRYPTS THE FILE USING A SYSTEM GENERATED KEY - STANDARD CLEAR OPERATION
		CLEARFile cf = CLEAR.clearFile();
		CLEARResult cr = cf.encrypt(dataToEncrypt.getAbsolutePath(), CLEAR.STRENGTH_512_BIT, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY);
		
		
		
		// PART 2 - RECOVER THE STANDARD KEY MATERIAL FROM THE ORIGINAL ENCRYPTION JOB
		String keyFile = cr.getKeyFile();
		String standardKeyMaterial = FileIO.getStringFromTextFile(new File(keyFile));
		
		
		
		// PART 3 - GENERATE HYPER-KEY FOR THIS OWNER - CONVERTS A STANDARD KEY INTO A HYPER KEY
		CLEARKeyTool keyTool = CLEAR.clearKeyTool();
		String hyperKey = keyTool.createHyperKey(standardKeyMaterial, ownerMFAMaterial);
		File hyperKeyFile = new File(workingDIR.getAbsolutePath() + File.separator + "hyperkey_original.ckey");
		FileIO.writeStringToFileWithOverwrite(hyperKeyFile, hyperKey);
		
		
		/**** OPTIONAL *****/
		// OPTIONAL FLOW - USER COULD START WITH ONLY A USK FIRST, (THUS AVOID "RE-ENCRYPT" STEP), OTHERWISE, ORIGINAL DATA NEEDS TO BE RE-ENCRYPTED WITH THE OWNER'S HYPERKEY AT THIS POINT
		File cipherTextFile = new File(cr.getCipherTextFile());
		File originalKeyFile = new File(cr.getKeyFile());
		cipherTextFile.delete();
		originalKeyFile.delete();
		/**** OPTIONAL *****/
		
		
		// TEST OWNER HYPERKEY!
		
		
		// PART 4 - ENCRYPT (OR RE-ENCRYPT) THE DATA WITH THE HYPERKEY
		String loadedOriginalHK = FileIO.getStringFromTextFile(hyperKeyFile);
		cf.encryptUSKWithMFA(dataToEncrypt.getAbsolutePath(), loadedOriginalHK, ownerMFAMaterial, StreamingEncryptionUserSuppliedKeyMode.STREAMING_USER_SUPPLIED_KEY);
		
		
//		Thread.sleep(2000);
//		dataToEncrypt.delete();
//		Thread.sleep(2000);
//		
//		cr = cf.decryptWithMFA(cr.getCipherTextFile(), hyperKeyFile.getAbsolutePath(), ownerMFAMaterial, StreamingDecryptionMode.STREAMING_KEY);
//		
//		
//		System.out.println("OBSERVE");
		
		
		
		// PART 4 - ADD USER-1 TO HYPER-KEY
		long len = dataToEncrypt.length();
		int user1Start1 = (int)len/2;
		int user1Stop1  = user1Start1 + ((int)len/4);
		int user1Start2 = user1Stop1 + ((int)len/6);
		int user1Stop2  = (int)(len -1);
		String updatedHyperKeyRound1 = keyTool.addHyperKeyACL(loadedOriginalHK, userToken1, ownerMFAMaterial, user1Start1, user1Stop1, user1Start2, user1Stop2);
		File hyperKeyFile1 = new File(workingDIR.getAbsolutePath() + File.separator + "hyperkey_with_user1.ckey");
		FileIO.writeStringToFileWithOverwrite(hyperKeyFile1, updatedHyperKeyRound1);
		
		
		
		// PART 5 - ADD USER-2 TO HYPER-KEY
		String loadedHK1 = FileIO.getStringFromTextFile(hyperKeyFile1);
		int user2Start1 = 1;
		int user2Stop1  = (int)len/2;
		int user2Start2 = (int)(len -1);
		int user2Stop2  = (int)(len -1);
		String updatedHyperKeyRound2 = keyTool.addHyperKeyACL(loadedHK1, userToken2, ownerMFAMaterial, user2Start1, user2Stop1, user2Start2, user2Stop2);
		File hyperKeyFile2 = new File(workingDIR.getAbsolutePath() + File.separator + "hyperkey_with_user2.ckey");
		FileIO.writeStringToFileWithOverwrite(hyperKeyFile2, updatedHyperKeyRound2);
		
		
		// PART 6 - DECRYPT FILE FROM OWNER PROFILE USING - HYPERKEY WITH 2 USERS
		dataToEncrypt.delete();
		Thread.sleep(2000); // remove original
		
		
		// TESTED OWNER FILE - [PASS - OK]
		cr = cf.decryptWithMFA(cipherTextFile.getAbsolutePath(), hyperKeyFile.getAbsolutePath(), ownerMFAMaterial, StreamingDecryptionMode.STREAMING_KEY);
		
		
		// TESTED USER-1 FILE - [PASS - OK]
		dataToEncrypt.delete();
		cr = cf.decryptWithMFA(cipherTextFile.getAbsolutePath(), hyperKeyFile1.getAbsolutePath(), ownerMFAMaterial, StreamingDecryptionMode.STREAMING_KEY);

		
		// TESTED USER-2 FILE - [PASS - OK]
		dataToEncrypt.delete();
		cr = cf.decryptWithMFA(cipherTextFile.getAbsolutePath(), hyperKeyFile2.getAbsolutePath(), ownerMFAMaterial, StreamingDecryptionMode.STREAMING_KEY);

		
		
		System.out.println("Done.");
		
		return pass;
	}
	
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			
			if ((args != null) && (args.length == 4) && (args[0] != null) && (args[1] != null) && (args[2] != null) && (args[3] != null)) {
				
				File mfa = new File(args[0]);
				File user1ACLToken = new File(args[1]);
				File user2ACLToken = new File(args[2]);
				File dataToEncrypt = new File(args[3]);
				
				if ((mfa != null) && (mfa.exists() 
				 && (user1ACLToken != null) && (user1ACLToken.exists())
				 && (user2ACLToken != null) && (user2ACLToken.exists())
				 && (dataToEncrypt != null) && (dataToEncrypt.exists()))) {
					
					
					// MAKE A BACKUP COPY OF THE INPUT FILE SO THAT IT CAN BE USED DURING TESTING
					byte[] ownerMFAMaterial = LoadMFA.load(mfa.getAbsolutePath());
					
					String userToken1 = FileIO.getStringFromTextFile(user1ACLToken);
					String userToken2 = FileIO.getStringFromTextFile(user2ACLToken);
					
					test(userToken1, userToken2, ownerMFAMaterial, dataToEncrypt); 	 // EXECUTE UNIT TEST
					
				}
				else {
					System.out.println("Could not locate OWNER MFA material or USER TOKENS:  " + args[0] + " - " + args[1] + "-" + args[2]);
				}
			}
			else {
				System.out.println("This unit test requires program input parameters");
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
