package tools;
/*********************************************************************************************
//* Filename:		Base64.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

public abstract class Base64 {

	final protected static char[] hexArray = "0123456789ABCDEF".toCharArray();
	
	/**
	 *  Encode String and return String.
	 */
	public final static String encode(String sS) {
		return encode(sS.getBytes());
	}

	/**
	 *  Encode some data and return a String.
	 */
	public final static byte[] encodeAsByteArray(byte[] d) {
		if (d == null)
			return null;
		byte data[] = new byte[d.length + 2];
		System.arraycopy(d, 0, data, 0, d.length);
		byte dest[] = new byte[(data.length / 3) * 4];

		// 3-byte to 4-byte conversion
		for (int sidx = 0, didx = 0; sidx < d.length; sidx += 3, didx += 4) {
			dest[didx] = (byte) ((data[sidx] >>> 2) & 077);
			dest[didx + 1] =
				(byte) ((data[sidx + 1] >>> 4) & 017 | (data[sidx] << 4) & 077);
			dest[didx + 2] =
				(byte) ((data[sidx + 2] >>> 6) & 003 | (data[sidx + 1] << 2) & 077);
			dest[didx + 3] = (byte) (data[sidx + 2] & 077);
		}

		// 0-63 to ascii printable conversion
		for (int idx = 0; idx < dest.length; idx++) {
			if (dest[idx] < 26)
				dest[idx] = (byte) (dest[idx] + 'A');
			else
				if (dest[idx] < 52)
					dest[idx] = (byte) (dest[idx] + 'a' - 26);
				else
					if (dest[idx] < 62)
						dest[idx] = (byte) (dest[idx] + '0' - 52);
					else
						if (dest[idx] < 63)
							dest[idx] = (byte) '+';
						else
							dest[idx] = (byte) '/';
		}

		// add padding
		for (int idx = dest.length - 1; idx > (d.length * 4) / 3; idx--) {
			dest[idx] = (byte) '=';
		}
		return dest;
	}
	
	/**
	 *  Encode some data and return a String.
	 */
	public final static String encode(byte[] d) {
		String rval = null;
		if (d != null) {
			rval = new String(encodeAsByteArray(d));
		}
		return rval;
	}

	/**
	 *  Decode data and return a String.
	 */
	public final static String decode2Str(String str) {
		if (str == null)
			return null;
		return new String(decode(str));
	}

	/**
	 *  Decode data and return bytes.
	 */
	public final static byte[] decode(String str) {
		if (str == null) {
			return null;
		}
		str = str + "=";
		byte data[] = str.getBytes();
		return decode(data);
	}

	/**
	 *  Decode data and return bytes.  Assumes that the data passed
	 *  in is ASCII text.
	 */
	public final static byte[] decode(byte[] data) {
		int tail = data.length;
		while (data[tail - 1] == '=')
			tail--;
		byte dest[] = new byte[tail - data.length / 4];

		// ascii printable to 0-63 conversion
		for (int idx = 0; idx < data.length; idx++) {
			if (data[idx] == '=')
				data[idx] = 0;
			else
				if (data[idx] == '/')
					data[idx] = 63;
				else
					if (data[idx] == '+')
						data[idx] = 62;
					else
						if (data[idx] >= '0' && data[idx] <= '9')
							data[idx] = (byte) (data[idx] - ('0' - 52));
						else
							if (data[idx] >= 'a' && data[idx] <= 'z')
								data[idx] = (byte) (data[idx] - ('a' - 26));
							else
								if (data[idx] >= 'A' && data[idx] <= 'Z')
									data[idx] = (byte) (data[idx] - 'A');
		}

		// 4-byte to 3-byte conversion
		int sidx, didx;
		for (sidx = 0, didx = 0; didx < dest.length - 2; sidx += 4, didx += 3) {
			dest[didx] = (byte) (((data[sidx] << 2) & 255) | ((data[sidx + 1] >>> 4) & 3));
			dest[didx + 1] =
				(byte) (((data[sidx + 1] << 4) & 255) | ((data[sidx + 2] >>> 2) & 017));
			dest[didx + 2] =
				(byte) (((data[sidx + 2] << 6) & 255) | (data[sidx + 3] & 077));
		}
		if (didx < dest.length) {
			dest[didx] = (byte) (((data[sidx] << 2) & 255) | ((data[sidx + 1] >>> 4) & 3));
		}
		if (++didx < dest.length) {
			dest[didx] =
				(byte) (((data[sidx + 1] << 4) & 255) | ((data[sidx + 2] >>> 2) & 017));
		}
		return dest;
	}
	
	/**
	 * Converts byte array to hex string
	 * @param bytes
	 * @return String
	*/
	public static String bytesToHex(byte[] bytes) {
		char[] hexChars = new char[bytes.length * 2];
		for ( int j = 0; j < bytes.length; j++ ) {
			int v = bytes[j] & 0xFF;
			hexChars[j * 2] = hexArray[v >>> 4];
			hexChars[j * 2 + 1] = hexArray[v & 0x0F];
		}
		return new String(hexChars);
	}
}
