package tools;
/*********************************************************************************************
//* Filename:		LoadMFA.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;
import java.io.RandomAccessFile;

public abstract class LoadMFA {

	/**
	 * UTILITY - LOAD BINARY FILE AS BYTE ARRAY
	 * @param fileName
	 * @return byte[]
	 * @throws Exception
	*/
	public static byte[] load(String fileName) throws Exception {
		
		byte[] rval = null;
		
		
		// CHECK FILE
		File file = new File(fileName);
		if (file.exists()) {
			
			RandomAccessFile inputRAF = null;
			
			try {
				
				// LOAD MFA DATA
				inputRAF = new RandomAccessFile(file, "r");
				inputRAF.seek(0L);
				final byte[] data = new byte[(int)file.length()];
			    inputRAF.readFully(data);
			    
			    rval = data;
				
			}
			catch(Exception ex) {
				ex.printStackTrace();
			}
			finally {
				try {
					if (inputRAF != null) {
						inputRAF.close();
					}
				}
				catch(Exception exx) {
					exx.printStackTrace();
					System.exit(1);
				}
			}
		}
		else {
			System.err.println("File not found: [" + fileName + "]");
		}
		
	    return rval;
	}
}
