package tools;
/*********************************************************************************************
//* Filename:		TestCleanup.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEARResult;

public abstract class TestCleanup {

	/**
	 * CLEANUP FILES CREATED BY PREVIOUS JOB
	 * @param cr
	 * @throws Exception
	*/
	public static void cleanup(CLEARResult cr) throws Exception {
		
		String ciphertextFileResult = cr.getCipherTextFile(); // EXAMINE RESULTS
		String keyFileResult = cr.getKeyFile(); 
		
		File ciphertextFile = new File(ciphertextFileResult);
		File keyFile = new File(keyFileResult);
		
		if (ciphertextFile.exists()) {
			ciphertextFile.delete();
		}
		
		if (keyFile.exists()) {
			keyFile.delete();
		}
		
		Thread.sleep(1000);  // WAIT [ONE] SECONDS TO OBSERVE THAT THE FILE HAS BEEN REMOVED - LOCALLY
	}
}
